@@ -1,7 +1,10 @@
 __docformat__ = 'google'
 
+import csv as csv_lib
+import io
 
-def _header_strings(die, include_weights, unpack_outcomes):
+
+def make_header(die, include_weights, unpack_outcomes):
     """Generates a list of strings for the header."""
     header = []
     if unpack_outcomes:
@@ -15,7 +18,7 @@ def _header_strings(die, include_weights, unpack_outcomes):
     return header, [len(s) for s in header]
 
 
-def _row_string(outcome, weight, p, include_weights, unpack_outcomes):
+def make_row(outcome, weight, p, include_weights, unpack_outcomes):
     row = []
     if unpack_outcomes:
         for x in outcome:
@@ -24,14 +27,14 @@ def _row_string(outcome, weight, p, include_weights, unpack_outcomes):
         row.append(str(outcome))
     if include_weights:
         row.append(str(weight))
-    row.append(f'{p:11.6%}')
+    row.append(f'{p:.6%}')
 
     return row
 
 
-def _row_strings(die, include_weights, unpack_outcomes):
+def make_rows(die, include_weights, unpack_outcomes):
     result = [
-        _row_string(outcome, weight, p, include_weights, unpack_outcomes)
+        make_row(outcome, weight, p, include_weights, unpack_outcomes)
         for outcome, weight, p in zip(die.outcomes(), die.weights(), die.pmf())
     ]
     col_widths = [max(len(s) for s in col) for col in zip(*result)]
@@ -51,26 +54,54 @@ def markdown(die, *, include_weights=True, unpack_outcomes=True):
         return 'Empty die\n'
 
     unpack_outcomes = unpack_outcomes and die.outcome_len() is not None
-    header_strings, header_widths = _header_strings(die, include_weights,
-                                                    unpack_outcomes)
-    row_strings, col_widths = _row_strings(die, include_weights,
-                                           unpack_outcomes)
+    header, header_widths = make_header(die, include_weights, unpack_outcomes)
+    rows, col_widths = make_rows(die, include_weights, unpack_outcomes)
     col_widths = [max(h, c) for h, c in zip(header_widths, col_widths)]
 
     result = f'Denominator: {die.denominator()}\n\n'
     result += '| '
     result += ' | '.join(
-        f'{s:>{width}}' for s, width in zip(header_strings, col_widths))
+        f'{s:>{width}}' for s, width in zip(header, col_widths))
     result += ' |\n'
 
     result += '|-'
     result += ':|-'.join('-' * width for width in col_widths)
     result += ':|\n'
 
-    for row in row_strings:
+    for row in rows:
         result += '| '
         result += ' | '.join(
             f'{s:>{width}}' for s, width in zip(row, col_widths))
         result += ' |\n'
 
     return result
+
+
+def csv(die,
+        *,
+        include_weights=True,
+        unpack_outcomes=True,
+        dialect='excel',
+        **fmtparams):
+    """Formats the die as a comma-separated-values string.
+
+    Args:
+        include_weights: If `True`, a column will be emitted for the weights.
+            Otherwise, only probabilities will be emitted.
+        unpack_outcomes: If `True` and all outcomes have a common length,
+            outcomes will be unpacked, producing one column per element.
+        dialect, **fmtparams: Will be sent to `csv.writer()`.
+    """
+
+    unpack_outcomes = unpack_outcomes and die.outcome_len() is not None
+    header, header_widths = make_header(die, include_weights, unpack_outcomes)
+    rows, col_widths = make_rows(die, include_weights, unpack_outcomes)
+    col_widths = [max(h, c) for h, c in zip(header_widths, col_widths)]
+
+    with io.StringIO() as out:
+        writer = csv_lib.writer(out, dialect=dialect, **fmtparams)
+        writer.writerow(header)
+        for row in rows:
+            writer.writerow(row)
+
+        return out.getvalue()
