@@ -10,14 +10,14 @@ import math
 
 
 class EvalPool(ABC):
-    """An abstract, immutable, callable class for evaulating one or more `DicePool`s.
+    """An abstract, immutable, callable class for evaulating one or more `Pool`s.
 
     There is one abstract method to implement: `next_state()`.
     This should incrementally calculate the result of the roll
     given one outcome at a time along with how many dice rolled that outcome.
     An example sequence of calls, as far as `next_state()` is concerned, is:
 
-    1. `state = next_state(state=None, 1, how_many_dice_rolled_1)`
+    1. `state = next_state(state=None, outcome=1, count=how_many_dice_rolled_1)`
     2. `state = next_state(state, 2, how_many_dice_rolled_2)`
     3. `state = next_state(state, 3, how_many_dice_rolled_3)`
     4. `state = next_state(state, 4, how_many_dice_rolled_4)`
@@ -86,7 +86,7 @@ class EvalPool(ABC):
 
         Args:
             final_state: A state after all outcomes have been processed.
-            *pools: One or more `DicePool`s being evaluated.
+            *pools: One or more `Pool`s being evaluated.
                 Most subclasses will expect a fixed number of pools and
                 can replace this variadic parameter with a fixed number of named
                 parameters.
@@ -100,13 +100,14 @@ class EvalPool(ABC):
     def direction(self, *pools):
         """Optional function to determine the direction in which `next_state()` will see outcomes.
 
-        Note that an ascending (> 0) direction is not compatible with pools with `truncate_min`,
-        and a descending (< 0) direction is not compatible with pools with `truncate_max`.
+        Note that an ascending (> 0) direction is not compatible with pools with
+        `truncate_min`, and a descending (< 0) direction is not compatible with
+        pools with `truncate_max`.
 
         The default is ascending order.
 
         Args:
-            *pools: One or more `DicePool`s being evaluated.
+            *pools: One or more `Pool`s being evaluated.
                 Most subclasses will expect a fixed number of pools and
                 can replace this variadic parameter with a fixed number of named
                 parameters.
@@ -131,7 +132,7 @@ class EvalPool(ABC):
 
         Args:
             *pools: Each element may be one of the following:
-                * A `DicePool` representing possible rolls of a pool.
+                * A `Pool` representing possible rolls of a pool.
                 * A dict-like representing a single roll of a pool.
                     The dict maps outcomes to counts.
                 * A sequence of outcomes representing a single roll of a pool.
@@ -139,17 +140,17 @@ class EvalPool(ABC):
                 Most evaluators will expect a fixed number of pools.
                 The outcomes of the pools must be mutually comparable.
                 Pools with `truncate_min` and pools with `truncate_max` are not
-                compatible.
+                compatible with each other.
 
         Returns:
             A die representing the distribution of the final score.
             If all pools are `PoolRoll`s, the result is a single outcome instead.
         """
 
-        # Cast non-pool arguments to `BasePool`.
+        # Cast non-pool arguments to `PoolBase`.
         pools = [
             pool
-            if isinstance(pool, icepool.BasePool) else icepool.PoolRoll(pool)
+            if isinstance(pool, icepool.PoolBase) else icepool.PoolRoll(pool)
             for pool in pools
         ]
 
@@ -223,7 +224,7 @@ class EvalPool(ABC):
 
         Arguments:
             direction: The direction in which to send outcomes to `next_state()`.
-            *pools: One or more `DicePool`s to evaluate.
+            *pools: One or more `Pool`s to evaluate.
                 This *does* change recursively.
 
         Returns:
@@ -300,7 +301,7 @@ def _pop_pool_max(outcome, pool):
 
     Args:
         outcome: The max outcome.
-        pool: The `DicePool` under consideration.
+        pool: The `Pool` under consideration.
 
     Yields:
         prev_pool: The remainder of the pool after taking out the dice that
@@ -320,7 +321,7 @@ def _pop_pool_min(outcome, pool):
 
     Args:
         outcome: The min outcome.
-        pool: The `DicePool` under consideration.
+        pool: The `Pool` under consideration.
 
     Yields:
         prev_pool: The remainder of the pool after taking out the dice that
@@ -336,7 +337,7 @@ def _pop_pool_min(outcome, pool):
 
 
 class WrapFuncEval(EvalPool):
-    """A `EvalPool` created from a single provided function.
+    """An `EvalPool` created from a single provided function.
 
     `next_state()` simply calls that function.
     """
@@ -353,6 +354,60 @@ class WrapFuncEval(EvalPool):
         return self._func(state, outcome, *counts)
 
 
+class JointEval(EvalPool):
+    """EXPERIMENTAL: An `EvalPool` that jointly evaluates subevals on the same roll(s) of a pool.
+
+    It may be more efficient to write the joint evaluation directly; this is
+    provided as a convenience.
+    """
+
+    def __init__(self, *subevals):
+        self._subevals = subevals
+
+    def next_state(self, state, outcome, *counts):
+        """Runs `next_state` for all subevals.
+
+        The state is a tuple of the substates.
+        """
+        if state is None:
+            return tuple(
+                subeval.next_state(None, outcome, *counts)
+                for subeval in self._subevals)
+        else:
+            return tuple(
+                subeval.next_state(substate, outcome, *counts)
+                for subeval, substate in zip(self._subevals, state))
+
+    def final_outcome(self, final_state, *pools):
+        """Runs `final_state` for all subevals.
+
+        The final outcome is a tuple of the final suboutcomes.
+        """
+        return tuple(
+            subeval.final_outcome(final_substate, *pools)
+            for subeval, final_substate in zip(self._subevals, final_state))
+
+    def direction(self, *pools):
+        """Determines the common direction of the subevals.
+
+        Raises:
+            ValueError: If subevals have conflicting directions, i.e. some are
+                ascending and others are descending.
+        """
+        subdirections = tuple(
+            subeval.direction(*pools) for subeval in self._subevals)
+        ascending = any(x > 0 for x in subdirections)
+        descending = any(x < 0 for x in subdirections)
+        if ascending and descending:
+            raise ValueError('Sub-evals have conflicting directions.')
+        elif ascending:
+            return 1
+        elif descending:
+            return -1
+        else:
+            return 0
+
+
 class SumPool(EvalPool):
     """A simple `EvalPool` that just sums the dice in a pool. """
 
@@ -368,8 +423,8 @@ class SumPool(EvalPool):
         return 0
 
 
-"""A shared `SumPool` object for caching results. """
 sum_pool = SumPool()
+"""A shared `SumPool` object for caching results. """
 
 
 class FindBestSet(EvalPool):
